/**
 * @file tlsplugin.c
 *
 *
 * This recognizes SOCKS protocol packets
 * http://en.wikipedia.org/wiki/SOCKS
 **
 ** ------------------------------------------------------------------------
 ** Copyright (C) 2007-2011 Carnegie Mellon University. All Rights Reserved.
 ** ------------------------------------------------------------------------
 ** Authors: Emily Sarneso <ecoff@cert.org>
 ** ------------------------------------------------------------------------
 ** @OPENSOURCE_HEADER_START@
 ** Use of the YAF system and related source code is subject to the terms 
 ** of the following licenses:
 ** 
 ** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
 ** Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
 ** 
 ** NO WARRANTY
 ** 
 ** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
 ** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
 ** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
 ** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
 ** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
 ** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
 ** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
 ** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
 ** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
 ** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
 ** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
 ** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
 ** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
 ** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
 ** DELIVERABLES UNDER THIS LICENSE.
 ** 
 ** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
 ** Mellon University, its trustees, officers, employees, and agents from 
 ** all claims or demands made against them (and any related losses, 
 ** expenses, or attorney's fees) arising out of, or relating to Licensee's 
 ** and/or its sub licensees' negligent use or willful misuse of or 
 ** negligent conduct or willful misconduct regarding the Software, 
 ** facilities, or other rights or assistance granted by Carnegie Mellon 
 ** University under this License, including, but not limited to, any 
 ** claims of product liability, personal injury, death, damage to 
 ** property, or violation of any laws or regulations.
 ** 
 ** Carnegie Mellon University Software Engineering Institute authored 
 ** documents are sponsored by the U.S. Department of Defense under 
 ** Contract FA8721-05-C-0003. Carnegie Mellon University retains 
 ** copyrights in all material produced under this contract. The U.S. 
 ** Government retains a non-exclusive, royalty-free license to publish or 
 ** reproduce these documents, or allow others to do so, for U.S. 
 ** Government purposes only pursuant to the copyright license under the 
 ** contract clause at 252.227.7013.
 ** 
 ** @OPENSOURCE_HEADER_END@    
 ** ------------------------------------------------------------------------
 *
 */

#define _YAF_SOURCE_
#include <yaf/autoinc.h>
#include <yaf/yafcore.h>
#include <yaf/decode.h>

#define SOCKS_PORT_NUMBER  1080


/**
 * socksplugin_LTX_ycSocksScanScan
 *
 * the scanner for recognizing SOCKS packets
 *
 * @param argc number of string arguments in argv
 * @param argv string arguments for this plugin (first two are library 
 *             name and function name)
 * @param payload the packet payload
 * @param payloadSize size of the packet payload
 * @param flow a pointer to the flow state structure
 * @param val a pointer to biflow state (used for forward vs reverse)
 *
 *
 * @return socks_port_number
 *         otherwise 0
 */
uint16_t
socksplugin_LTX_ycSocksScanScan (
    int argc,
    char *argv[],
    uint8_t * payload,
    unsigned int payloadSize,
    yfFlow_t * flow,
    yfFlowVal_t * val)
{

    uint8_t offsetptr = 0;
    uint16_t socks_port;
    uint32_t socks_ip;
    uint8_t num_auth_methods = 0;
    uint8_t auth_method;
    int loop;

    if (payloadSize < 2) {
        return 0;
    }

    if (payload[0] == 4) {
	
	if (payload[1] != 1 && payload[1] != 2) {
	    return 0;
	}

	offsetptr += 2;

	if (offsetptr + 6 > payloadSize) {
	    return 0;
	}

	socks_port = ntohs(*(uint16_t *)(payload + offsetptr));
	offsetptr += 2;
	socks_ip = ntohl(*(uint32_t *)(payload + offsetptr));
	
	if (socks_ip != flow->key.addr.v4.dip) {
	    if (socks_ip > 0xFF) {
		return 0;
	    }
	}
	
    } else if (payload[0] == 5) {
	num_auth_methods = payload[1];
	
	if ((num_auth_methods + 2) > payloadSize) {
	    return 0;
	}
	offsetptr += 2;
	for (loop = 0; loop < num_auth_methods; loop++) {
	    auth_method = *(payload + offsetptr + loop);
	    if (auth_method == 4 || auth_method > 9) {
		/* not assigned */
		return 0;
	    }
	}
	offsetptr += loop;
	
	if (offsetptr == payloadSize) {
	    return SOCKS_PORT_NUMBER;
	}

	if ((*(payload + offsetptr) != 5)) {
	    return 0;
	} 

    } else {
	return 0;
    }
    return SOCKS_PORT_NUMBER;
}
